require "behaviours/wander"
require "behaviours/chaseandattack"
require "behaviours/follow"
require "behaviours/doaction"
require "behaviours/minperiod"
require "behaviours/panic"
require "behaviours/runaway"


SEE_PLAYER_DIST = 10
local SEE_DIST = 30
local TOOCLOSE = 6
local SEE_FOOD_DIST = 8
local MIN_FOLLOW = 10
local MAX_FOLLOW = 20
local MED_FOLLOW = 15

local MIN_RUNAWAY = 8
local MAX_RUNAWAY = MED_FOLLOW


local function StealAction(inst)
    if not inst.components.inventory:IsFull() then
		local player = GetPlayer()
		local target = FindEntity(inst, SEE_DIST, function(item) 
			if item.components.inventoryitem and 
				item.components.inventoryitem.canbepickedup and 
				not item.components.inventoryitem:IsHeld() and
				item:IsOnValidGround() and 
				not item:HasTag("irreplaceable") and
				not item:HasTag("prey") and
				not item:HasTag("bird") and
				not item:HasTag("bottle") and
				not item:HasTag("water_sourse")
				then
				
					return player and player:GetDistanceSqToInst(item) > TOOCLOSE*TOOCLOSE
				end
			end)
	    
		if target then
			return BufferedAction(inst, target, ACTIONS.PICKUP)
		end
	end
end


local function EmptyChest(inst)
    if not inst.components.inventory:IsFull() then
		local player = GetPlayer()
		local target = FindEntity(inst, SEE_DIST, function(item) 
			if item.prefab == "treasurechest" and 
				item.components.container and
				not item.components.container:IsEmpty() then
					return player and player:GetDistanceSqToInst(item) > TOOCLOSE*TOOCLOSE
				end
			end)
		if target then
			return BufferedAction(inst, target, ACTIONS.HAMMER)
		end
	end
end

local MIN_FOLLOW = 10
local MAX_FOLLOW = 20
local MED_FOLLOW = 15

local MIN_RUNAWAY = 8
local MAX_RUNAWAY = MED_FOLLOW

local darkclonestealbrain = Class(Brain, function(self, inst)
    Brain._ctor(self, inst)
    self.greed = 2 + math.random(4)
end)
local function FindNearestBush(inst)
    local bush = FindEntity(inst, SEE_BUSH_DIST, function(bush) return bush.components.pickable and bush.components.pickable:CanBePicked() end, {"bush"})
    return bush or (inst.components.homeseeker and inst.components.homeseeker.home)
end

local function HomePos(inst)
    local bush = FindNearestBush(inst)
    if bush then
        return Vector3(bush.Transform:GetWorldPosition() )
    end
end

local function GoHomeAction(inst)
    local bush = FindNearestBush(inst)
    if bush then
        return BufferedAction(inst, bush, ACTIONS.GOHOME, nil, Vector3(bush.Transform:GetWorldPosition() ) )
    end
end

local function EatFoodAction(inst)
    local target = nil
    if inst.components.inventory and inst.components.eater then
        target = inst.components.inventory:FindItem(function(item) return inst.components.eater:CanEat(item) end)
    end
    if not target then
        target = FindEntity(inst, SEE_FOOD_DIST, function(item) return inst.components.eater:CanEat(item) end)
        if target then
            --check for scary things near the food
            local predator = GetClosestInstWithTag("scarytoprey", target, SEE_PLAYER_DIST)
            if predator then target = nil end
        end
    end
    if target then
        local act = BufferedAction(inst, target, ACTIONS.EAT)
        act.validfn = function() return not (target.components.inventoryitem and target.components.inventoryitem.owner and target.components.inventoryitem.owner ~= inst) end
        return act
    end
end

local function PickBerriesAction(inst)
    local target = FindEntity(inst, SEE_FOOD_DIST, function(item)
        return item.components.pickable
               and item.components.pickable:CanBePicked()
               and item.components.pickable.product == "berries"
    end)
    if target then
        --check for scary things near the bush
        local predator = GetClosestInstWithTag("scarytoprey", target, SEE_PLAYER_DIST)
        if predator then target = nil end
    end
    if target then
        return BufferedAction(inst, target, ACTIONS.PICK)
    end
end


function darkclonestealbrain:OnStart()
    
    local stealnode = PriorityNode(
	{
		DoAction(self.inst, function() return StealAction(self.inst) end, "steal", true ),        
		DoAction(self.inst, function() return EmptyChest(self.inst) end, "emptychest", true )
	}, 2)

    
    local root = PriorityNode(
    {
        WhileNode( function() return self.inst.components.health.takingfiredamage end, "OnFire", Panic(self.inst)),
        ChaseAndAttack(self.inst, 100),
		
		
		DoAction(self.inst, PickBerriesAction, "Pick Berries", true),

		
		IfNode( function() return self.inst.components.inventory:NumItems() >= self.greed and not self.inst.sg:HasStateTag("busy") end, "donestealing",
			ActionNode(function() self.inst.sg:GoToState("exit") return SUCCESS end, "leave" )),
		MinPeriod(self.inst, 10, 
			stealnode),

        RunAway(self.inst, "player", MIN_RUNAWAY, MAX_RUNAWAY),
		Follow(self.inst, function() return GetPlayer() end, MIN_FOLLOW, MED_FOLLOW, MAX_FOLLOW),
        --Wander(self.inst, function() local player = GetPlayer() if player then return Vector3(player.Transform:GetWorldPosition()) end end, 20, true)
    }, 2)
    
    self.bt = BT(self.inst, root)
   
end

return darkclonestealbrain