local GalunMonsterSpawner = Class(function(self, inst)
    self.inst = inst
    self.inst:StartUpdatingComponent(self)
    self.monsters = {}
    self.shadowhands = {}

	self.popchangetimer = nil
	self.currenttargetpop = 0
	self.currentpop = 0
	self.spawntimer = nil
	
	self.num_galun = 0
	self.num_galun_item = 0
	self.num_galun_str = 0
	self.num_darkclone =0
	
	self.num_creepy_eyes = 0
	self.num_watchers = 0
	self.num_skitters = 0
	self.nowatchertime = 10*math.random()
	
	self.nogaluntime = 7*math.random()
	self.nogaluntime_item = 3*math.random()
	self.nogaluntime_str = 3*math.random()
	

	self.darkclone = nil
	self.darkclone_timeout = 0
	
	
	
	self.noeyestime = 20*math.random()
	self.noskittersstime = 20*math.random()
	self.creepyhandtimer = TUNING.SEG_TIME*20
end)

local GALUN = {"spider","spider", "spider", "hound", "icehound","leif",
"spider_warrior", "merm","merm", "firehound","knight_nightmare","leif","frog_galun",
"rook_galun","rook_nightmare_galun"
}
local GALUN_SW = {"spider","spider", "spider", "hound", "icehound","leif",
"spider_warrior", "merm","merm", "firehound","knight_nightmare","leif","frog_galun",
"rook_galun","rook_nightmare_galun", "snake"
}
local GALUN_SW_WATER = {"swordfish", "sharx", "sharx", "sharx", "stungray_galun"}


local GALUNITEMS = {"silk", "axe", "spear", "goldnugget","log","bluegem",
"redgem","purplegem","log","log","axe","goldnugget","shovel"}


local GALUNSTR = {"pond", "evergreen", "berrybush","berrybush2","reeds","grass","grass","evergreen","grass","evergreen","rock1","rock2"}
local GALUNSTR_ROG = {"pond", "deciduoustree", "berrybush","berrybush2","reeds","grass","grass","evergreen","grass","evergreen","rock1","rock2"}
local GALUNSTR_SW = {"palmtree","jungletree","palmtree","jungletree","berrybush","berrybush2","reeds","grass","grass","grass","rock1","rock2"}
local GALUNSTR_SW_WATER = {"seaweed_planted","coralreef", nil, nil, nil }


local GALUN_FOOD = {"carrot", "berries", "seeds", "seeds","seeds","meat","smallmeat"}
local GALUN_HEALTH = {"mosquitosack", "spidergland", "petals", "petals","petals", "petals"}



local GALUNSANITY_START = 0.5





function GalunMonsterSpawner:OnSave()
    if self.noserial then return end
    
    local refs = {}
    local data = {
		monsters={},
		shadowhands = {},
		currenttargetpop = self.currenttargetpop,
		popchangetimer = self.popchangetimer,
		spawntimer = self.spawntimer,
		creepyhandtimer = self.creepyhandtimer,
		num_darkclone=self.num_darkclone
	}

    for k,v in pairs(self.monsters) do
		table.insert(data.monsters, v.GUID)
		table.insert(refs, v.GUID)
    end

    for k,v in pairs(self.shadowhands) do
		table.insert(data.shadowhands, v.GUID)
		table.insert(refs, v.GUID)
    end
	
    return data, refs
end


function GalunMonsterSpawner:OnLoad(data)
	self.popchangetimer = data.popchangetimer
	self.currenttargetpop = data.currenttargetpop or 0
	self.spawntimer = data.spawntimer
	self.creepyhandtimer = data.creepyhandtimer or self.creepyhandtimer
	
	---find clone
	if data.num_darkclone and data.num_darkclone>0 then
	
		local clone=FindEntity(self.inst, 20, function(guy) return (guy:HasTag("darkclone"))end)
		if clone then
			self.num_darkclone=1
		end
		
	end
end




function GalunMonsterSpawner:OnProgress()
	self.noserial = true
end



function GalunMonsterSpawner:LoadPostPass(newents, savedata)
    local num = 0
    if savedata.monsters then
        for k,v in pairs(savedata.monsters) do
            local child = newents[v]
            if child and child.entity:HasTag("shadowcreature") then
				num = num + 1
				child = child.entity
				table.insert(self.monsters, child)
				child:ListenForEvent( "onremove", function() self:OnMonsterRemoved( child ) end, child )	
            end
        end
    end
    
    if savedata.shadowhands then
        for k,v in pairs(savedata.shadowhands) do
            local child = newents[v]
            if child and child.entity:HasTag("shadowhand") then
				child = child.entity
				table.insert(self.shadowhands, child)
				child:ListenForEvent( "onremove", function() self:ShadowHandRemoved( child ) end, child )	
            end
        end
    end
    
    self.currentpop = num
end



function GalunMonsterSpawner:FindSpawnPoint()
	local radius = 15
	local theta = math.random()*2*PI
	local offset = Vector3(radius * math.cos( theta ), 0, -radius * math.sin( theta ) )
	local spawnPos = Vector3(self.inst.Transform:GetWorldPosition()) + offset  
	return spawnPos
end






------------------------------------------------------------------------------------------------------------------------
------------------------------------------------------------------------------------------------------------------------
------------------------------------------------------------------------------------------------------------------------
------------------------------------------------------------------------------------------------------------------------
------------------------------------------------------------------------------------------------------------------------
------------------------------------------------------------------------------------------------------------------------


local function destroy_galun(inst, times)
	if inst.components.combat then inst.components.combat:GiveUp() end
	local time_to_erode = times or 1 + math.random()*3
	
	local tick_time = TheSim:GetTickTime()

	if inst.DynamicShadow then
        inst.DynamicShadow:Enable(false)
    end

	inst:StartThread( function()
		local ticks = 0
		while ticks * tick_time < time_to_erode do
			local erode_amount = ticks * tick_time / time_to_erode
			inst.AnimState:SetErosionParams( erode_amount, 0.1, 1.0 )
			ticks = ticks + 1
			Yield()
		end
		inst:Remove()
	end)
end


local function onsave(inst, data)
	if inst.removeOnLoad then
		data.removeOnLoad = true
	end
end

local function onload(inst, data)
	if data and data.removeOnLoad then
		inst:Remove()
	end
end

local function postinitremoveonload(inst)  
	inst.OnSave = onsave
	inst.OnLoad = onload
end





local function FindGalunItemSpawnPoint(dist,light,self)
    local dst = dist or 7
	local radius = dst + math.random()*9
	local theta = math.random()*2*PI
	local offset = Vector3(radius * math.cos( theta ), 0, -radius * math.sin( theta ) )
	local spawnPos = Vector3(GetPlayer().Transform:GetWorldPosition()) + offset  
    local lght = light or 0.17
	local lightLevel = TheSim:GetLightAtPoint(spawnPos:Get() )
	if lightLevel <= lght then
		return spawnPos
    elseif math.random(1,5) > 2 then
		FindGalunItemSpawnPoint(dist+1,light-0.01, self)
	end
end

local function FindGalunItemSpawnPointWater(dist,light,self)
    local dst = dist or 7
	local radius = dst + math.random()*9
	local theta = math.random()*2*PI
	local offset = Vector3(radius * math.cos( theta ), 0, -radius * math.sin( theta ) )
    
	local spawnPos = Vector3(GetPlayer().Transform:GetWorldPosition()) + offset

	
	local lght = light or 0.17
	local lightLevel = TheSim:GetLightAtPoint(spawnPos:Get() )
	if lightLevel <= lght then
		return spawnPos
    elseif math.random(1,5) > 2 then
		FindGalunItemSpawnPointWater(dist+1,light-0.01, self)
	end
end

local function destroy(inst)
	local time_to_erode = 1
	local tick_time = TheSim:GetTickTime()

	if inst.DynamicShadow then
        inst.DynamicShadow:Enable(false)
    end

	inst:StartThread( function()
		local ticks = 0
		while ticks * tick_time < time_to_erode do
			local erode_amount = ticks * tick_time / time_to_erode
			inst.AnimState:SetErosionParams( erode_amount, 0.1, 1.0 )
			ticks = ticks + 1
			Yield()
		end
		inst:Remove()
	end)
end



function GalunMonsterSpawner:GalunMobAttacker(spawn_interval,self,count,target)
	local dist= {8,11,14}
	local distwater= {5,7,9}
	
	local target1 = target
	if target1 == nil then
		target1 = GetPlayer()
	else
		dist= {8,11,14}
		distwater= {5,7,9}
	end

	
	local i = 0
	repeat

		i = i + 1
		local pt = nil
		if target1 == GetPlayer() then
			if self.inst:HasTag("aquatic") then
				if GetClock():IsNight() then
					pt = FindGalunItemSpawnPointWater(distwater[1],0.17,self)
				elseif GetClock():IsDusk() then
					pt = FindGalunItemSpawnPointWater(distwater[2],0.7,self)
				else
					pt = FindGalunItemSpawnPointWater(distwater[3],10,self)
				end			
			else
				if GetClock():IsNight() then
					pt = FindGalunItemSpawnPoint(dist[1],0.17,self)
				elseif GetClock():IsDusk() then
					pt = FindGalunItemSpawnPoint(dist[2],0.7,self)
				else
					pt = FindGalunItemSpawnPoint(dist[3],10,self)
				end
			end
		else
			local theta = math.random()*2*PI
			local offset = Vector3(2 * math.cos( theta ), 0, -2 * math.sin( theta ) )
			pt = Vector3(target1.Transform:GetWorldPosition()) + offset  
			  
		end				
		if pt and GetWorld().Map:GetTileAtPoint(pt:Get()) ~= GROUND.IMPASSABLE then
			local prefab_tospwn = GALUN[math.random(#GALUN)]
			
			if self.inst:HasTag("aquatic") then
				prefab_tospwn = GALUN_SW_WATER[math.random(#GALUN_SW_WATER)] 
			elseif self.inst.components.driver then
				prefab_tospwn = GALUN_SW[math.random(#GALUN_SW)] 
			end
			
			local monster = SpawnPrefab(prefab_tospwn)

			if monster then
				monster.Transform:SetPosition(pt:Get() )
				if self.num_galun == nil then
					self.num_galun = 5
				end
				
				monster:ListenForEvent("onremove", function() self.num_galun = self.num_galun - 1 end, monster )
				monster.components.combat:SetTarget(target1)
				
				
				local lightwatch = monster.entity:AddLightWatcher()
				lightwatch:SetLightThresh(.2)
				lightwatch:SetDarkThresh(.05)
				
				monster.components.health:SetMaxHealth(100)
				monster.components.combat:SetDefaultDamage(monster.components.combat.defaultdamage*0.07)
				
				if target1 == GetPlayer() then
					self.nogaluntime = spawn_interval + math.random()*spawn_interval*.5
					self.num_galun = self.num_galun + 1
					if math.random() < 0.05 then
						local lightning = SpawnPrefab("lightning")
						lightning.Transform:SetPosition(monster.Transform:GetWorldPosition())
					end
				else
					GetPlayer().SoundEmitter:PlaySound("dontstarve/sanity/creature1/appear","galun_sound")
					GetPlayer():DoTaskInTime(1.8, function()
					GetPlayer().SoundEmitter:KillSound("galun_sound")
					end)
				
					monster.components.locomotor.runspeed = 6
					monster.components.locomotor.walkspeed = 6

					SpawnPrefab("collapse_big").Transform:SetPosition(monster.Transform:GetWorldPosition()) --the titular poof
					monster:AddTag("sword_attacker")
					
					monster.components.combat:SetDefaultDamage(monster.components.combat.defaultdamage)
					monster.AnimState:SetMultColour(.7, .7, .7, 0.4)

					monster:DoTaskInTime(math.random()*6+3, function()
						destroy_galun(monster, math.random()*3+1)
					end)
				end
				
				monster:AddTag("shadow")
				monster:AddTag("galun")
				monster.components.combat.playerdamagepercent = 0
				monster:AddComponent("galun")
				
				monster:AddComponent("eater")
				monster.components.eater:SetInsectivore()
				
				
				monster:RemoveComponent("lootdropper")
				monster:AddComponent("lootdropper")
				
				--if math.random(1,7) == 2 then
					-----monster.components.lootdropper:AddRandomLoot("nightmarefuel", 1)
				--end
					
				monster.components.lootdropper:AddRandomLoot("petals_evil",1)
				monster.components.lootdropper.numrandomloot = math.floor(math.random(0,0))
				
				monster.persists=false
				monster.removeOnLoad = true
				
				monster:DoTaskInTime(4, function()
				destroy(monster)
				end)
				

				end			
								
		end
	until i > count
end


function GalunMonsterSpawner:UpdateGalun(dt)
	self.darkclone_timeout = self.darkclone_timeout - dt
	local sanity = self.inst.components.sanity:GetPercent()
	if not GetClock():IsDay() and sanity <= .4 then
		if self.nogaluntime and self.nogaluntime > 0 then
			self.nogaluntime = self.nogaluntime - dt
		else
		
			local max_galun = 3
			local spawn_interval = 21
			
			max_galun = math.floor(7-(sanity*9))
			local spawn_interval = math.floor(6+(sanity*12))

			if self.num_galun < max_galun then
			self:GalunMobAttacker(spawn_interval,self,math.floor(math.random(1,max_galun-self.num_galun)),nil)
			else
			self.num_galun = 2
			
end
	end
end
end

local function GalunStrItem(spawn_interval,self,count,itemorstr)

	local i = 0
    repeat

	i = i + 1


	local pt = nil
	if not self.inst:HasTag("aquatic") then
		if GetClock():IsNight() then
			pt = FindGalunItemSpawnPoint(8,0.17,self)
		elseif GetClock():IsDusk() then
			pt = FindGalunItemSpawnPoint(14,0.45,self)
		else
			pt = FindGalunItemSpawnPoint(17,1,self)
		end
	else
		if GetClock():IsNight() then
			pt = FindGalunItemSpawnPointWater(7,0.17,self)
		elseif GetClock():IsDusk() then
			pt = FindGalunItemSpawnPointWater(9,0.45,self)
		else
			pt = FindGalunItemSpawnPointWater(11,1,self)
		end
	end

	
	if pt and GetWorld().Map:GetTileAtPoint(pt:Get()) ~= GROUND.IMPASSABLE then				
		local prefab_tospwn = nil
		
		if itemorstr == true then
			if self.inst:HasTag("aquatic") then
				prefab_tospwn = GALUNITEMS[math.random(#GALUNITEMS)]				
			else
				prefab_tospwn = GALUNITEMS[math.random(#GALUNITEMS)]
				if GetPlayer().components.hunger:GetPercent() < 0.6 and math.random(0,(GetPlayer().components.hunger:GetPercent()))<0.2 then
					prefab_tospwn = GALUN_FOOD[math.random(#GALUN_FOOD)]
					--print("spawn galun food.."..prefab_tospwn)
				end
				if GetPlayer().components.health:GetPercent() < 0.6 and math.random(0,(GetPlayer().components.health:GetPercent()))<0.15 then
					prefab_tospwn = GALUN_HEALTH[math.random(#GALUN_HEALTH)]
					--print("spawn galun health.."..prefab_tospwn)
				end
			end
		elseif self.inst:HasTag("aquatic") then
			prefab_tospwn = GALUNSTR_SW_WATER[math.random(#GALUNSTR_SW_WATER)]
		elseif self.inst.components.driver then
			prefab_tospwn = GALUNSTR_SW[math.random(#GALUNSTR_SW)]		
		elseif self.inst.components.moisture then
			prefab_tospwn = GALUNSTR_ROG[math.random(#GALUNSTR_ROG)]
		else
			prefab_tospwn = GALUNSTR[math.random(#GALUNSTR)]
		end
					
		local monster = SpawnPrefab(prefab_tospwn)
		
		if monster then
			monster.Transform:SetPosition(pt:Get() )
							
			if itemorstr == true then
				monster:ListenForEvent("onremove", function() self.num_galun_item = self.num_galun_item - 1 end, monster )
				self.num_galun_item = self.num_galun_item + 1
				monster.components.inventoryitem.canbepickedup = false
			else
				monster:ListenForEvent("onremove", function() self.num_galun_str = self.num_galun_str - 1 end, monster )
				self.num_galun_str = self.num_galun_str + 1

				if monster.components.childspawner then
					monster.components.childspawner:StopRegen()
					monster.components.childspawner.childname = ""
					monster.components.childspawner.rarechild = nil
				end
			end
						
						
			self.nogaluntime = spawn_interval + math.random()*spawn_interval*.5
			local lightwatch = monster.entity:AddLightWatcher()
			lightwatch:SetLightThresh(.2)
			lightwatch:SetDarkThresh(.05)
							
			monster:AddTag("galun")
			monster:AddComponent("galun")
			monster.persists=false
			
			monster:DoTaskInTime(math.random()*15+5, function()
				destroy_galun(monster, math.random()*3+3)
			end)				
		end			
	end
until i > count
end



function GalunMonsterSpawner:UpdateGalunItem(dt)
	local sanity = self.inst.components.sanity:GetPercent()

	if sanity <= .45 then
			--	self.inst.HUD.controls.mapcontrols.minimapBtn:Hide()
	else
			--	self.inst.HUD.controls.mapcontrols.minimapBtn:Show()

end
	if not GetClock():IsDay() and sanity <= .55 then

		if self.nogaluntime_item and self.nogaluntime_item > 0 then
			self.nogaluntime_item = self.nogaluntime_item - dt
		else
		
			local max_galun = math.floor(7-(sanity*10))
			local spawn_interval = math.floor(6+(sanity*9))


			if self.num_galun_item < max_galun then
			GalunStrItem(spawn_interval,self,math.floor(math.random(1,max_galun-self.num_galun_item)),true)

		end
	end
end
end




function GalunMonsterSpawner:UpdateGalunStr(dt)
	local sanity = self.inst.components.sanity:GetPercent()
	
	if not GetClock():IsDay() and sanity <= .53 then
		if self.nogaluntime_str and self.nogaluntime_str > 0 then
			self.nogaluntime_str = self.nogaluntime_str - dt
		else
		
			local max_galun = math.floor(10-(sanity*10))
			local spawn_interval = math.floor(4+(sanity*7))

			if self.num_galun_str < max_galun then
			GalunStrItem(spawn_interval,self,math.floor(math.random(1,max_galun-self.num_galun_str)),false)
		end
	end
end
end


function GalunMonsterSpawner:CreateDarkClone()

	if self.inst:HasTag("aquatic") then return end

	local sanity = self.inst.components.sanity:GetPercent()
	local colour = 1-(sanity*2)
	local pt = self:FindSpawnPoint()----FindGalunItemSpawnPoint(10,0.7,self)
	
	if not pt then
		print("NO PT CANT SPAWN")
	
	return end
	print("CLONE CREATED")
	local darkclone = SpawnPrefab("darkclone")
	darkclone.Transform:SetPosition(pt:Get() )
	self.darkclone = darkclone
	self.inst.components.galunmonsterspawner.darkclone_timeout = 10
end

function GalunMonsterSpawner:FindClone()
	if self.darkclone == nil then
		self.darkclone = FindEntity(self.inst, 100, function(guy) return (guy:HasTag("darkclone"))end)
	end
	return self.darkclone
end

function GalunMonsterSpawner:UpdateDarkClone(dt)

	local sanity = self.inst.components.sanity:GetPercent()
	local colour = 1-(sanity*2)
	---spawn func
	if GetClock():IsNight() and sanity <= .55 and (self.num_darkclone <1) then
			local pt = FindGalunItemSpawnPoint(17,1,self)
				
				if pt and GetWorld().Map:GetTileAtPoint(pt:Get()) ~= GROUND.IMPASSABLE then

					local darkclone = SpawnPrefab("darkclone")
					
					
					darkclone.Transform:SetPosition(pt:Get() )
					darkclone:ListenForEvent("onremove", function() self.num_darkclone = self.num_darkclone - 1 end, darkclone )
					self.num_darkclone = self.num_darkclone + 1

	if not GetClock():IsDay() then
	   darkclone.AnimState:SetMultColour(colour,colour,colour,colour*2)
	else
	   darkclone.AnimState:SetMultColour(0.1,.1,.1,0.1)
    end
	
        local brain2 = require "brains/darkclonestealbrain"
        darkclone:SetBrain(brain2)
		local brain2 = require "brains/darkclonebrain"
	    darkclone:SetBrain(brain2)


		end
		
	end

end


----------
function GalunMonsterSpawner:OnUpdate(dt)
	
	self:UpdateGalun(dt)
	self:UpdateGalunItem(dt)
	self:UpdateGalunStr(dt)
	----self:UpdateDarkClone(dt)
		
end

function GalunMonsterSpawner:LongUpdate(dt)
	self:OnUpdate(dt)
end

return GalunMonsterSpawner
